@@ -2,7 +2,8 @@ import fnmatch
 import json
 import os
 import re
-from typing import Any, Dict, List, Optional
+import sys
+from typing import Any, Optional
 
 from zrb.builtin.llm.tool.sub_agent import create_sub_agent_tool
 from zrb.config.config import CFG
@@ -10,7 +11,17 @@ from zrb.config.llm_rate_limitter import llm_rate_limitter
 from zrb.context.any_context import AnyContext
 from zrb.util.file import read_file, read_file_with_line_numbers, write_file
 
-_EXTRACT_INFO_FROM_FILE_SYSTEM_PROMPT = CFG.LLM_ANALYZE_FILE_EXTRACTOR_SYSTEM_PROMPT
+if sys.version_info >= (3, 12):
+    from typing import TypedDict
+else:
+    from typing_extensions import TypedDict
+
+
+class FileToWrite(TypedDict):
+    """Represents a file to be written, with a 'path' and 'content'."""
+
+    path: str
+    content: str
 
 
 DEFAULT_EXCLUDED_PATTERNS = [
@@ -218,7 +229,7 @@ def read_from_file(
 
     Returns:
         str: A JSON object containing the file path, the requested content with line numbers, the start and end lines, and the total number of lines in the file.
-             Example: '{"path": "src/main.py", "content": "1: import os\n2: \n3: print(\"Hello, World!\")", "start_line": 1, "end_line": 3, "total_lines": 3}'
+             Example: '{"path": "src/main.py", "content": "1| import os\n2| \n3| print(\"Hello, World!\")", "start_line": 1, "end_line": 3, "total_lines": 3}'
     Raises:
         FileNotFoundError: If the specified file does not exist.
     """
@@ -471,7 +482,7 @@ async def analyze_file(
     _analyze_file = create_sub_agent_tool(
         tool_name="analyze_file",
         tool_description="analyze file with LLM capability",
-        system_prompt=CFG.LLM_ANALYZE_FILE_EXTRACTOR_SYSTEM_PROMPT,
+        system_prompt=CFG.LLM_FILE_EXTRACTOR_SYSTEM_PROMPT,
         tools=[read_from_file, search_files],
     )
     payload = json.dumps(
@@ -481,18 +492,18 @@ async def analyze_file(
     return await _analyze_file(ctx, clipped_payload)
 
 
-def read_many_files(paths: List[str]) -> str:
+def read_many_files(paths: list[str]) -> str:
     """
     Reads and returns the full content of multiple files at once.
 
     This tool is highly efficient for gathering context from several files simultaneously. Use it when you need to understand how different files in a project relate to each other, or when you need to inspect a set of related configuration or source code files.
 
     Args:
-        paths (List[str]): A list of paths to the files you want to read. It is crucial to provide accurate paths. Use the `list_files` tool first if you are unsure about the exact file locations.
+        paths (list[str]): A list of paths to the files you want to read. It is crucial to provide accurate paths. Use the `list_files` tool first if you are unsure about the exact file locations.
 
     Returns:
         str: A JSON object where keys are the file paths and values are their corresponding contents, prefixed with line numbers. If a file cannot be read, its value will be an error message.
-             Example: '{"results": {"src/api.py": "1: import ...", "config.yaml": "1: key: value"}}'
+             Example: '{"results": {"src/api.py": "1| import ...", "config.yaml": "1| key: value"}}'
     """
     results = {}
     for path in paths:
@@ -507,7 +518,7 @@ def read_many_files(paths: List[str]) -> str:
     return json.dumps({"results": results})
 
 
-def write_many_files(files: Dict[str, str]) -> str:
+def write_many_files(files: list[FileToWrite]) -> str:
     """
     Writes content to multiple files in a single, atomic operation.
 
@@ -516,7 +527,7 @@ def write_many_files(files: Dict[str, str]) -> str:
     Each file's content is completely replaced. If a file does not exist, it will be created. If it exists, its current content will be entirely overwritten. Therefore, you must provide the full, intended content for each file.
 
     Args:
-        files (Dict[str, str]): A dictionary where keys are the file paths and values are the complete contents to be written to those files.
+        files: A list of file objects, where each object is a dictionary containing a 'path' and the complete 'content'.
 
     Returns:
         str: A JSON object summarizing the operation, listing successfully written files and any files that failed, along with corresponding error messages.
@@ -524,8 +535,13 @@ def write_many_files(files: Dict[str, str]) -> str:
     """
     success = []
     errors = {}
-    for path, content in files.items():
+    # 4. Access the data using dictionary key-lookup syntax.
+    for file in files:
         try:
+            # Use file['path'] and file['content'] instead of file.path
+            path = file["path"]
+            content = file["content"]
+
             abs_path = os.path.abspath(os.path.expanduser(path))
             directory = os.path.dirname(abs_path)
             if directory and not os.path.exists(directory):
@@ -535,6 +551,3 @@ def write_many_files(files: Dict[str, str]) -> str:
         except Exception as e:
             errors[path] = f"Error writing file: {e}"
     return json.dumps({"success": success, "errors": errors})
-
-
-apply_diff = replace_in_file
