@@ -1,4 +1,5 @@
 import datetime
+import sys
 from typing import Any
 
 from zrb.config.config import CFG
@@ -26,6 +27,7 @@ class SharedContext(AnySharedContext):
         env: dict[str, str] = {},
         xcom: dict[str, Xcom] = {},
         logging_level: int | None = None,
+        is_web_mode: bool = False,
     ):
         self.__logging_level = logging_level
         self._input = DotDict(input)
@@ -34,6 +36,7 @@ class SharedContext(AnySharedContext):
         self._xcom = DotDict(xcom)
         self._session: AnySession | None = None
         self._log = []
+        self._is_web_mode = is_web_mode
 
     def __repr__(self):
         class_name = self.__class__.__name__
@@ -43,6 +46,17 @@ class SharedContext(AnySharedContext):
         xcom = self._xcom
         return f"<{class_name} input={input} args={args} xcom={xcom} env={env}>"
 
+    @property
+    def is_web_mode(self) -> bool:
+        return self._is_web_mode
+
+    @property
+    def is_tty(self) -> bool:
+        try:
+            return sys.stdin.isatty()
+        except Exception:
+            return False
+
     @property
     def input(self) -> DotDict:
         return self._input
