@@ -1,14 +1,27 @@
-from typing import TYPE_CHECKING, Any, Callable
+from typing import TYPE_CHECKING, Callable
 
 if TYPE_CHECKING:
     from pydantic_ai.models import Model
     from pydantic_ai.settings import ModelSettings
 
-from zrb.attr.type import StrAttr, fstring
+from zrb.attr.type import BoolAttr, StrAttr, fstring
 from zrb.config.llm_config import LLMConfig, llm_config
 from zrb.context.any_context import AnyContext
 from zrb.context.any_shared_context import AnySharedContext
-from zrb.util.attr import get_attr
+from zrb.util.attr import get_attr, get_bool_attr
+
+
+def get_is_yolo_mode(
+    ctx: AnyContext,
+    is_yolo_mode_attr: BoolAttr | None = None,
+    render_yolo_mode: bool = True,
+):
+    return get_bool_attr(
+        ctx,
+        is_yolo_mode_attr,
+        llm_config.default_yolo_mode,
+        auto_render=render_yolo_mode,
+    )
 
 
 def get_model_settings(
