@@ -2,7 +2,7 @@ import json
 from collections.abc import Callable
 from typing import TYPE_CHECKING, Any
 
-from zrb.attr.type import BoolAttr, IntAttr, StrAttr, fstring
+from zrb.attr.type import BoolAttr, IntAttr, StrAttr, StrListAttr, fstring
 from zrb.config.llm_rate_limitter import LLMRateLimiter
 from zrb.context.any_context import AnyContext
 from zrb.context.any_shared_context import AnySharedContext
@@ -12,22 +12,19 @@ from zrb.task.any_task import AnyTask
 from zrb.task.base_task import BaseTask
 from zrb.task.llm.agent import get_agent, run_agent_iteration
 from zrb.task.llm.config import (
+    get_is_yolo_mode,
     get_model,
     get_model_settings,
 )
-from zrb.task.llm.context import extract_default_context
-from zrb.task.llm.context_enrichment import maybe_enrich_context
-from zrb.task.llm.history import (
-    ConversationHistoryData,
-    ListOfDict,
+from zrb.task.llm.conversation_history import (
     read_conversation_history,
     write_conversation_history,
 )
+from zrb.task.llm.conversation_history_model import ConversationHistory
 from zrb.task.llm.history_summarization import maybe_summarize_history
 from zrb.task.llm.prompt import (
-    get_combined_system_prompt,
-    get_context_enrichment_prompt,
-    get_summarization_prompt,
+    get_summarization_system_prompt,
+    get_system_and_user_prompt,
     get_user_message,
 )
 from zrb.util.cli.style import stylize_faint
@@ -35,9 +32,9 @@ from zrb.xcom.xcom import Xcom
 
 if TYPE_CHECKING:
     from pydantic_ai import Agent, Tool
-    from pydantic_ai.mcp import MCPServer
     from pydantic_ai.models import Model
     from pydantic_ai.settings import ModelSettings
+    from pydantic_ai.toolsets import AbstractToolset
 
     ToolOrCallable = Tool | Callable
 else:
@@ -67,47 +64,49 @@ class LLMTask(BaseTask):
         ) = None,
         agent: "Agent | Callable[[AnySharedContext], Agent] | None" = None,
         persona: StrAttr | None = None,
+        render_persona: bool = False,
         system_prompt: StrAttr | None = None,
+        render_system_prompt: bool = False,
         special_instruction_prompt: StrAttr | None = None,
+        render_special_instruction_prompt: bool = False,
+        modes: StrListAttr | None = None,
+        render_modes: bool = True,
         message: StrAttr | None = None,
         render_message: bool = True,
-        enrich_context: BoolAttr | None = None,
-        render_enrich_context: bool = True,
-        context_enrichment_prompt: StrAttr | None = None,
-        render_context_enrichment_prompt: bool = True,
-        context_enrichment_token_threshold: IntAttr | None = None,
-        render_context_enrichment_token_threshold: bool = True,
         tools: (
             list["ToolOrCallable"]
             | Callable[[AnySharedContext], list["ToolOrCallable"]]
         ) = [],
-        mcp_servers: (
-            list["MCPServer"] | Callable[[AnySharedContext], list["MCPServer"]]
+        toolsets: (
+            list["AbstractToolset[Agent]"] | Callable[[AnySharedContext], list["Tool"]]
         ) = [],
         conversation_history: (
-            ConversationHistoryData
-            | Callable[[AnySharedContext], ConversationHistoryData | dict | list]
+            ConversationHistory
+            | Callable[[AnySharedContext], ConversationHistory | dict | list]
             | dict
             | list
-        ) = ConversationHistoryData(),
+        ) = ConversationHistory(),
         conversation_history_reader: (
-            Callable[[AnySharedContext], ConversationHistoryData | dict | list | None]
+            Callable[[AnySharedContext], ConversationHistory | dict | list | None]
             | None
         ) = None,
         conversation_history_writer: (
-            Callable[[AnySharedContext, ConversationHistoryData], None] | None
+            Callable[[AnySharedContext, ConversationHistory], None] | None
         ) = None,
         conversation_history_file: StrAttr | None = None,
         render_history_file: bool = True,
         summarize_history: BoolAttr | None = None,
         render_summarize_history: bool = True,
         summarization_prompt: StrAttr | None = None,
+        render_summarization_prompt: bool = False,
         history_summarization_token_threshold: IntAttr | None = None,
         render_history_summarization_token_threshold: bool = True,
         rate_limitter: LLMRateLimiter | None = None,
         execute_condition: bool | str | Callable[[AnySharedContext], bool] = True,
         retries: int = 2,
         retry_period: float = 0,
+        is_yolo_mode: bool | None = None,
+        render_yolo_mode: bool = True,
         readiness_check: list[AnyTask] | AnyTask | None = None,
         readiness_check_delay: float = 0.5,
         readiness_check_period: float = 5,
@@ -152,24 +151,22 @@ class LLMTask(BaseTask):
         self._model_settings = model_settings
         self._agent = agent
         self._persona = persona
+        self._render_persona = render_persona
         self._system_prompt = system_prompt
+        self._render_system_prompt = render_system_prompt
         self._special_instruction_prompt = special_instruction_prompt
+        self._render_special_instruction_prompt = render_special_instruction_prompt
+        self._modes = modes
+        self._render_modes = render_modes
         self._message = message
         self._render_message = render_message
         self._summarization_prompt = summarization_prompt
-        self._should_enrich_context = enrich_context
-        self._render_enrich_context = render_enrich_context
-        self._context_enrichment_prompt = context_enrichment_prompt
-        self._render_context_enrichment_prompt = render_context_enrichment_prompt
-        self._context_enrichment_token_threshold = context_enrichment_token_threshold
-        self._render_context_enrichment_token_threshold = (
-            render_context_enrichment_token_threshold
-        )
+        self._render_summarization_prompt = render_summarization_prompt
         self._tools = tools
         self._rate_limitter = rate_limitter
         self._additional_tools: list["ToolOrCallable"] = []
-        self._mcp_servers = mcp_servers
-        self._additional_mcp_servers: list["MCPServer"] = []
+        self._toolsets = toolsets
+        self._additional_toolsets: list["AbstractToolset[Agent]"] = []
         self._conversation_history = conversation_history
         self._conversation_history_reader = conversation_history_reader
         self._conversation_history_writer = conversation_history_writer
@@ -185,6 +182,8 @@ class LLMTask(BaseTask):
         )
         self._max_call_iteration = max_call_iteration
         self._conversation_context = conversation_context
+        self._is_yolo_mode = is_yolo_mode
+        self._render_yolo_mode = render_yolo_mode
 
     def add_tool(self, *tool: ToolOrCallable):
         self.append_tool(*tool)
@@ -193,18 +192,12 @@ class LLMTask(BaseTask):
         for single_tool in tool:
             self._additional_tools.append(single_tool)
 
-    def add_mcp_server(self, *mcp_server: "MCPServer"):
-        self.append_mcp_server(*mcp_server)
+    def add_toolset(self, *toolset: "AbstractToolset[Agent]"):
+        self.append_toolset(*toolset)
 
-    def append_mcp_server(self, *mcp_server: "MCPServer"):
-        for single_mcp_server in mcp_server:
-            self._additional_mcp_servers.append(single_mcp_server)
-
-    def set_should_enrich_context(self, enrich_context: bool):
-        self._should_enrich_context = enrich_context
-
-    def set_context_enrichment_token_threshold(self, enrichment_token_threshold: int):
-        self._context_enrichment_token_threshold = enrichment_token_threshold
+    def append_toolset(self, *toolset: "AbstractToolset[Agent]"):
+        for single_toolset in toolset:
+            self._additional_toolsets.append(single_toolset)
 
     def set_should_summarize_history(self, summarize_history: bool):
         self._should_summarize_history = summarize_history
@@ -226,55 +219,69 @@ class LLMTask(BaseTask):
             model_api_key_attr=self._model_api_key,
             render_model_api_key=self._render_model_api_key,
         )
-        context_enrichment_prompt = get_context_enrichment_prompt(
+        is_yolo_mode = get_is_yolo_mode(
             ctx=ctx,
-            context_enrichment_prompt_attr=self._context_enrichment_prompt,
+            is_yolo_mode_attr=self._is_yolo_mode,
+            render_yolo_mode=self._render_yolo_mode,
         )
-        summarization_prompt = get_summarization_prompt(
+        summarization_prompt = get_summarization_system_prompt(
             ctx=ctx,
             summarization_prompt_attr=self._summarization_prompt,
+            render_summarization_prompt=self._render_summarization_prompt,
         )
         user_message = get_user_message(ctx, self._message, self._render_message)
-        # Get the combined system prompt using the new getter
-        system_prompt = get_combined_system_prompt(
-            ctx=ctx,
-            persona_attr=self._persona,
-            system_prompt_attr=self._system_prompt,
-            special_instruction_prompt_attr=self._special_instruction_prompt,
-        )
         # 1. Prepare initial state (read history from previous session)
-        history_data = await read_conversation_history(
+        conversation_history = await read_conversation_history(
             ctx=ctx,
             conversation_history_reader=self._conversation_history_reader,
             conversation_history_file_attr=self._conversation_history_file,
             render_history_file=self._render_history_file,
             conversation_history_attr=self._conversation_history,
         )
-        history_list = history_data.history
-        long_term_context = history_data.long_term_context
-        conversation_summary = history_data.conversation_summary
-
-        # 2. Enrich context and summarize history sequentially
-        new_long_term_context = await maybe_enrich_context(
+        conversation_history.fetch_newest_notes()
+        # 2. Get system prompt and user prompt
+        system_prompt, user_message = get_system_and_user_prompt(
             ctx=ctx,
-            history_list=history_list,
-            long_term_context=long_term_context,
-            should_enrich_context_attr=self._should_enrich_context,
-            render_enrich_context=self._render_enrich_context,
-            context_enrichment_token_threshold_attr=self._context_enrichment_token_threshold,
-            render_context_enrichment_token_threshold=self._render_context_enrichment_token_threshold,  # noqa
-            model=model,
-            model_settings=model_settings,
-            context_enrichment_prompt=context_enrichment_prompt,
-            rate_limitter=self._rate_limitter,
+            user_message=user_message,
+            persona_attr=self._persona,
+            render_persona=self._render_persona,
+            system_prompt_attr=self._system_prompt,
+            render_system_prompt=self._render_system_prompt,
+            special_instruction_prompt_attr=self._special_instruction_prompt,
+            render_special_instruction_prompt=self._render_special_instruction_prompt,
+            modes_attr=self._modes,
+            render_modes=self._render_modes,
+            conversation_history=conversation_history,
         )
-        new_history_list, new_conversation_summary = await maybe_summarize_history(
+        # 3. Get the agent instance
+        agent = get_agent(
             ctx=ctx,
-            history_list=history_list,
-            conversation_summary=conversation_summary,
+            agent_attr=self._agent,
+            model=model,
+            system_prompt=system_prompt,
+            model_settings=model_settings,
+            tools_attr=self._tools,
+            additional_tools=self._additional_tools,
+            toolsets_attr=self._toolsets,
+            additional_toolsets=self._additional_toolsets,
+            is_yolo_mode=is_yolo_mode,
+        )
+        # 4. Run the agent iteration and save the results/history
+        result = await self._execute_agent(
+            ctx,
+            agent,
+            user_message,
+            conversation_history,
+        )
+        # 5. Summarize
+        conversation_history = await maybe_summarize_history(
+            ctx=ctx,
+            conversation_history=conversation_history,
             should_summarize_history_attr=self._should_summarize_history,
             render_summarize_history=self._render_summarize_history,
-            history_summarization_token_threshold_attr=self._history_summarization_token_threshold,  # noqa
+            history_summarization_token_threshold_attr=(
+                self._history_summarization_token_threshold
+            ),
             render_history_summarization_token_threshold=(
                 self._render_history_summarization_token_threshold
             ),
@@ -283,50 +290,22 @@ class LLMTask(BaseTask):
             summarization_prompt=summarization_prompt,
             rate_limitter=self._rate_limitter,
         )
-
-        # 3. Build the final user prompt and system prompt
-        final_user_prompt, system_info = extract_default_context(user_message)
-        context_parts = [
-            f"## System Information\n{json.dumps(system_info, indent=2)}",
-        ]
-        if new_long_term_context:
-            context_parts.append(new_long_term_context)
-        if new_conversation_summary:
-            context_parts.append(new_conversation_summary)
-
-        final_system_prompt = "\n\n".join(
-            [system_prompt, "# Context", "\n\n---\n\n".join(context_parts)]
-        )
-        # 4. Get the agent instance
-        agent = get_agent(
+        # 6. Write conversation history
+        await write_conversation_history(
             ctx=ctx,
-            agent_attr=self._agent,
-            model=model,
-            system_prompt=final_system_prompt,
-            model_settings=model_settings,
-            tools_attr=self._tools,
-            additional_tools=self._additional_tools,
-            mcp_servers_attr=self._mcp_servers,
-            additional_mcp_servers=self._additional_mcp_servers,
-        )
-        # 5. Run the agent iteration and save the results/history
-        return await self._run_agent_and_save_history(
-            ctx,
-            agent,
-            final_user_prompt,
-            new_history_list,
-            new_long_term_context,
-            new_conversation_summary,
+            history_data=conversation_history,
+            conversation_history_writer=self._conversation_history_writer,
+            conversation_history_file_attr=self._conversation_history_file,
+            render_history_file=self._render_history_file,
         )
+        return result
 
-    async def _run_agent_and_save_history(
+    async def _execute_agent(
         self,
         ctx: AnyContext,
         agent: "Agent",
         user_prompt: str,
-        history_list: ListOfDict,
-        long_term_context: str,
-        conversation_summary: str,
+        conversation_history: ConversationHistory,
     ) -> Any:
         """Executes the agent, processes results, and saves history."""
         try:
@@ -334,23 +313,12 @@ class LLMTask(BaseTask):
                 ctx=ctx,
                 agent=agent,
                 user_prompt=user_prompt,
-                history_list=history_list,
+                history_list=conversation_history.history,
                 rate_limitter=self._rate_limitter,
             )
             if agent_run and agent_run.result:
                 new_history_list = json.loads(agent_run.result.all_messages_json())
-                data_to_write = ConversationHistoryData(
-                    long_term_context=long_term_context,
-                    conversation_summary=conversation_summary,
-                    history=new_history_list,
-                )
-                await write_conversation_history(
-                    ctx=ctx,
-                    history_data=data_to_write,
-                    conversation_history_writer=self._conversation_history_writer,
-                    conversation_history_file_attr=self._conversation_history_file,
-                    render_history_file=self._render_history_file,
-                )
+                conversation_history.history = new_history_list
                 xcom_usage_key = f"{self.name}-usage"
                 if xcom_usage_key not in ctx.xcom:
                     ctx.xcom[xcom_usage_key] = Xcom([])
